unit Main;
  { (c) 2008, Tom Verhoeff (TU/e) }
  { Graphical User Interface }

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    InputGroupBox: TGroupBox;
    OutputGroupBox: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure Button1Click(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation

uses
  Lists; { unit under test }
  
const
  Digits = [ '0' .. '9' ];

procedure EatNonDigitsFromStr ( var t: String );
  { pre:  True
    post: t = old t without leading non-digits }
begin

  while (t <> '') and not (t[1] in Digits) do begin
    Delete ( t, 1, 1 );
  end; { while }
  {@ (t = '') or (t[1] in Digits) }

end;

procedure EatIntFromStr ( var t: String; out v: Integer );
  { pre:  t[1] in Digits
    post: t = old t without leading digits;
          v = value of leading number in t (0 <= v) }
begin
  v := 0;

  while (t <> '') and (t[1] in Digits) do begin
    v := 10 * v + StrToInt ( t[1] );
    Delete ( t, 1, 1);
  end; { while }
  {@ (t = '') or not (t[1] in Digits) }
  
end;

procedure StrToList ( const t: String; out s: TList );
  { convert string to list }
  { pre:  True
    post: s = list encoded in t }
var
  v: Integer; { to capture result from EatIntFromStr }
  tc: String; { to process t }
begin
  tc := Copy ( t, 1, Length(t) );
  with s do begin
    len := 0;
    EatNonDigitsFromStr ( tc );
    {@ inv: (tc = '') or (tc[1] in Digits) }
    
    while tc <> '' do begin
      {@ (tc[1] in Digits }
      EatIntFromStr ( tc, v );
      val[len] := v mod (High(TEntry) + 1); { mod for robustness }
      Inc(len);
      if len = MaxListLen then begin { for robustness }
        tc := '';
      end; { if }
      EatNonDigitsFromStr ( tc );
      {@ (tc = '') or (tc[1] in Digits) }
    end; { while }
    {@ tc = '' }
    
  end; { with s }
end;

function ListToStr ( const s: TList ): String;
  { pre:  True
    ret:  string representation of s }
var
  i: Integer; { to traverse s.val }
  sep: String; { separator }
begin
  with s do begin
    Result := '';
    sep := '';
  
    for i := 0 to len - 1 do begin
      Result := Result + sep + IntToStr ( val[i] );
      sep := ' ';
    end; { for i }

  end; { with s }
end;

{ TForm1 }

procedure TForm1.Button1Click(Sender: TObject);
var
  s: TList; { the list }
  x: TEntry; { value to find }
  found: Boolean; { to capture result of Find }
  pos: TIndex; { to capture result of Find }
  t: String; { to manipulate text }
  v: Integer; { to capture result of EatIntFromStr }
begin
  with Edit1 do begin
    StrToList ( Text, s );
    Text := ListToStr ( s );
  end; { with Edit1 }
  with Edit2 do begin { for robustness }
    t := Text;
    EatNonDigitsFromStr ( t );
    if (t <> '') and (t[1] in Digits) then begin
      EatIntFromStr ( t, v );
      x := v mod (High(TEntry) + 1);
    end
    else begin
      x := 0;
    end;
    Text := IntToStr ( x );
  end; { with Edit2 }
  
  Find ( s, x, found, pos );

  if found then begin
    Edit3.Text := 'True';
  end
  else begin
    Edit3.Text := 'False';
  end;
  if found then begin
    Edit4.Text := IntToStr ( pos );
  end
  else begin
    Edit4.Text := '';
  end;
end;

initialization
  {$I Main.lrs}

end.

